#ifndef SMALLCHANGE_BASIC_H
#define SMALLCHANGE_BASIC_H

/**************************************************************************\
 * Copyright (c) Kongsberg Oil & Gas Technologies AS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
\**************************************************************************/

/* *********************************************************************** */

/*
  NOTE: basic.h is automatically generated from basic.h.cmake.in, so don't
  edit basic.h directly.
*/

/* *********************************************************************** */

/* A unique identifier to recognize Systems in Motion's SmallChange
 * library.
 */
#define __SMALLCHANGE__

/* The following #undef'ed defines are automatically defined and
 * synchronized with the settings in configure.in when ``configure''
 * is executed.
 *
 * The #ifndef wrapper is necessary because during development, these
 * are also defined in the config.h file generated by ``configure''.
 */
#ifndef SMALLCHANGE_VERSION

/* These are available for adding or omitting features based on
 * SmallChange version numbers in client application sources. */
#define SMALLCHANGE_MAJOR_VERSION 1
#define SMALLCHANGE_MINOR_VERSION 0
#define SMALLCHANGE_MICRO_VERSION 0
#define SMALLCHANGE_VERSION 1.0.0

#endif /* !SMALLCHANGE_VERSION */ /* Ends configure-generated defines. */


/* *********************************************************************** */

/* Precaution to avoid an some errors easily made by the application
   programmer. */
#ifdef SMALLCHANGE_DLL_API
# error Leave the internal SMALLCHANGE_DLL_API define alone.
#endif /* SMALLCHANGE_DLL_API */
#ifdef SMALLCHANGE_INTERNAL
# ifdef SMALLCHANGE_NOT_DLL
#  error The SMALLCHANGE_NOT_DLL define is not supposed to be used when building the library, only when building Win32 applications.
# endif /* SMALLCHANGE_INTERNAL && SMALLCHANGE_NOT_DLL */
# ifdef SMALLCHANGE_DLL
#  error The SMALLCHANGE_DLL define is not supposed to be used when building the library, only when building Win32 applications.
# endif /* SMALLCHANGE_INTERNAL && SMALLCHANGE_DLL */
#endif /* SMALLCHANGE_INTERNAL */

/*
  On MSWindows platforms, one of these defines must always be set when
  building application programs:

   - "SMALLCHANGE_DLL", when the application programmer is using the
     library in the form of a dynamic link library (DLL)

   - "SMALLCHANGE_NOT_DLL", when the application programmer is using
     the library in the form of a static object library (LIB)

  Note that either SMALLCHANGE_DLL or SMALLCHANGE_NOT_DLL _must_ be
  defined by the application programmer on MSWindows platforms, or
  else the #error statement will hit. Set up one or the other of these
  two defines in your compiler environment according to how the
  library was built -- as a DLL (use "SMALLCHANGE_DLL") or as a LIB
  (use "SMALLCHANGE_NOT_DLL").

  (Setting up defines for the compiler is typically done by either
  adding something like "/DSMALLCHANGE_DLL" to the compiler's argument
  line (for command-line build processes), or by adding the define to
  the list of preprocessor symbols in your IDE GUI (in the MSVC IDE,
  this is done from the "Project"->"Settings" menu, choose the "C/C++"
  tab, then "Preprocessor" from the dropdown box and add the
  appropriate define)).

  It is extremely important that the application programmer uses the
  correct define, as using "SMALLCHANGE_NOT_DLL" when
  "SMALLCHANGE_DLL" is correct will cause mysterious crashes.
 */
/* FIXME: use a feature check to see if this is a platform which can
   recognize the __declspec keyword instead of the crap #if below.
   20011201 mortene. */
#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__)
# ifdef SMALLCHANGE_INTERNAL
#  ifdef SMALLCHANGE_MAKE_DLL
#   define SMALLCHANGE_DLL_API __declspec(dllexport)
#  endif /* SMALLCHANGE_MAKE_DLL */
# else /* !SMALLCHANGE_INTERNAL */
#  ifdef SMALLCHANGE_DLL
#   ifdef SMALLCHANGE_NOT_DLL
#     error Define _either_ SMALLCHANGE_DLL _or_ SMALLCHANGE_NOT_DLL as appropriate for your linkage -- not both at the same time! See Inventor/C/basic.h for further instructions.
#   endif /* SMALLCHANGE_NOT_DLL */
#   define SMALLCHANGE_DLL_API __declspec(dllimport)
#  else /* !SMALLCHANGE_DLL */
#   ifndef SMALLCHANGE_NOT_DLL
#    error Define either SMALLCHANGE_DLL or SMALLCHANGE_NOT_DLL as appropriate for your linkage! See Inventor/C/basic.h for further instructions.
#   endif /* SMALLCHANGE_NOT_DLL */
#  endif /* !SMALLCHANGE_DLL */
# endif /* !SMALLCHANGE_INTERNAL */
#endif /* Microsoft Windows */

/* Empty define to avoid errors when _not_ compiling an MSWindows DLL. */
#ifndef SMALLCHANGE_DLL_API
# define SMALLCHANGE_DLL_API
#endif /* !SMALLCHANGE_DLL_API */


#endif /* !SMALLCHANGE_BASIC_H */
