# CMake package configuration file for SmallChange
#
# Defines the target "SmallChange::SmallChange"
#
# Add the following lines to your CMakeLists.txt to depend on SmallChange
#
#    find_package(SmallChange REQUIRED)
#    target_link_libraries(my_target_name SmallChange::SmallChange)
#
# Additionally you may one of the following variables (or their corresponding
# upper case version) that are also defined.
#
# SmallChange_COMPILE_DEFINITIONS
# SmallChange_DEFINITIONS
# SmallChange_INCLUDE_DIRS
# SmallChange_INCLUDE_DIR
# SmallChange_LIBRARY
# SmallChange_LIBRARIES
# SmallChange_LIBRARY_DIRS
# SmallChange_LIBRARY_DIR
# SmallChange_DOC_DIR
#
# However, in most cases using the SmallChange::SmallChange target is sufficient,
# and you won't need these variables.


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was smallchange-config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

include(CMakeFindDependencyMacro)

find_dependency(Coin REQUIRED)

set(_have_qt5 1)
set(_have_qt4 )
if(_have_qt5)
  find_dependency(Qt5Core REQUIRED)
  find_dependency(Qt5Widgets REQUIRED)
elseif(_have_qt4)
  find_dependency(Qt4 REQUIRED)
endif()

include("${CMAKE_CURRENT_LIST_DIR}/smallchange-export.cmake")

get_property(SmallChange_COMPILE_DEFINITIONS TARGET SmallChange::SmallChange PROPERTY INTERFACE_COMPILE_DEFINITIONS)
foreach(_def ${SmallChange_COMPILE_DEFINITIONS})
  list(APPEND SmallChange_DEFINITIONS -D${_def})
endforeach()

set(SmallChange_VERSION 1.0.0)

get_property(SmallChange_INCLUDE_DIRS TARGET SmallChange::SmallChange PROPERTY INTERFACE_INCLUDE_DIRECTORIES)
set(SmallChange_INCLUDE_DIR ${SmallChange_INCLUDE_DIRS})
set(SmallChange_LIBRARY SmallChange::SmallChange)
get_property(SmallChange_LIBRARIES TARGET SmallChange::SmallChange PROPERTY INTERFACE_LINK_LIBRARIES)
set(SmallChange_LIBRARIES SmallChange::SmallChange ${SmallChange_LIBRARIES})

set_and_check(SmallChange_LIBRARY_DIRS "${PACKAGE_PREFIX_DIR}/lib")
set(SmallChange_LIBRARY_DIR ${SmallChange_LIBRARY_DIRS})

# optional directory from documentation component
set_and_check(SmallChange_DOC_DIR "${PACKAGE_PREFIX_DIR}/share/doc/SmallChange")

# For backwards compatibility define upper case versions of output variables
foreach(_var
  SmallChange_COMPILE_DEFINITIONS
  SmallChange_DEFINITIONS
  SmallChange_INCLUDE_DIRS
  SmallChange_INCLUDE_DIR
  SmallChange_LIBRARY
  SmallChange_LIBRARIES
  SmallChange_LIBRARY_DIRS
  SmallChange_LIBRARY_DIR
  SmallChange_DOC_DIR
  SmallChange_VERSION
  )
  string(TOUPPER ${_var} _uppercase_var)
  set(${_uppercase_var} ${${_var}})
endforeach()
